from hmac import new
from hashlib import sha1
from base64 import encodestring

from django.conf import settings
from django.http import HttpResponse
from django.test import Client, RequestFactory, TestCase
from django.test.utils import override_settings

from twilio.twiml import Response

from django_twilio import settings as django_twilio_settings
from django_twilio.tests.views import response_view, str_view, verb_view


class TwilioViewTestCase(TestCase):
    fixtures = ['django_twilio.json']
    urls = 'django_twilio.tests.urls'

    def setUp(self):
        self.client = Client(enforce_csrf_checks=True)
        self.factory = RequestFactory(enforce_csrf_checks=True)

        # Test URIs.
        self.uri = 'http://testserver/tests/decorators'
        self.str_uri = '/tests/decorators/str_view/'
        self.verb_uri = '/tests/decorators/verb_view/'
        self.response_uri = '/tests/decorators/response_view/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        django_twilio_settings.TWILIO_ACCOUNT_SID = 'xxx'
        django_twilio_settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate Twilio signatures for our test views.
        self.response_signature = encodestring(
            new(django_twilio_settings.TWILIO_AUTH_TOKEN,
                '%s/response_view/' % self.uri, sha1).digest()).strip()
        self.str_signature = encodestring(
            new(django_twilio_settings.TWILIO_AUTH_TOKEN,
                '%s/str_view/' % self.uri, sha1).digest()).strip()
        self.sig_with_from_field_normal_caller = encodestring(
            new(django_twilio_settings.TWILIO_AUTH_TOKEN,
                '%s/str_view/From+12222222222' % self.uri,
                sha1).digest()).strip()
        self.sig_with_from_field_blacklisted_caller = encodestring(
            new(django_twilio_settings.TWILIO_AUTH_TOKEN,
                '%s/str_view/From+13333333333' % self.uri,
                sha1).digest()).strip()
        self.verb_signature = encodestring(
            new(django_twilio_settings.TWILIO_AUTH_TOKEN,
                '%s/verb_view/' % self.uri, sha1).digest()).strip()

    def test_requires_post(self):
        debug_orig = settings.DEBUG
        settings.DEBUG = False
        self.assertEquals(self.client.get(self.str_uri).status_code, 405)
        self.assertEquals(self.client.head(self.str_uri).status_code, 405)
        self.assertEquals(self.client.options(self.str_uri).status_code, 405)
        self.assertEquals(self.client.put(self.str_uri).status_code, 405)
        self.assertEquals(self.client.delete(self.str_uri).status_code, 405)
        settings.DEBUG = True
        self.assertEquals(self.client.get(self.str_uri).status_code, 200)
        self.assertEquals(self.client.head(self.str_uri).status_code, 200)
        self.assertEquals(self.client.options(self.str_uri).status_code, 200)
        self.assertEquals(self.client.put(self.str_uri).status_code, 200)
        self.assertEquals(self.client.delete(self.str_uri).status_code, 200)
        settings.DEBUG = debug_orig

    def test_allows_post(self):
        request = self.factory.post(
            self.str_uri, HTTP_X_TWILIO_SIGNATURE=self.str_signature)
        self.assertEquals(str_view(request).status_code, 200)

    def test_decorator_preserves_metadata(self):
        self.assertEqual(str_view.__name__, 'str_view')

    def test_missing_settings_return_forbidden(self):
        del django_twilio_settings.TWILIO_ACCOUNT_SID
        del django_twilio_settings.TWILIO_AUTH_TOKEN
        debug_orig = settings.DEBUG
        settings.DEBUG = False
        self.assertEquals(self.client.post(self.str_uri).status_code, 403)
        settings.DEBUG = True
        self.assertEquals(self.client.post(self.str_uri).status_code, 200)
        settings.DEBUG = debug_orig

    def test_missing_signature_returns_forbidden(self):
        debug_orig = settings.DEBUG
        settings.DEBUG = False
        self.assertEquals(self.client.post(self.str_uri).status_code, 403)
        settings.DEBUG = True
        self.assertEquals(self.client.post(self.str_uri).status_code, 200)
        settings.DEBUG = debug_orig

    def test_incorrect_signature_returns_forbidden(self):
        debug_orig = settings.DEBUG
        settings.DEBUG = False
        request = self.factory.post(
            self.str_uri, HTTP_X_TWILIO_SIGNATURE='fakesignature')
        self.assertEquals(str_view(request).status_code, 403)
        settings.DEBUG = True
        self.assertEquals(str_view(request).status_code, 200)
        settings.DEBUG = debug_orig

    def test_no_from_field(self):
        request = self.factory.post(
            self.str_uri,
            HTTP_X_TWILIO_SIGNATURE=self.str_signature)
        self.assertEquals(str_view(request).status_code, 200)

    def test_from_field_no_caller(self):
        request = self.factory.post(
            self.str_uri, {'From': '+12222222222'},
            HTTP_X_TWILIO_SIGNATURE=self.sig_with_from_field_normal_caller)
        self.assertEquals(str_view(request).status_code, 200)

    def test_blacklist_works(self):
        debug_orig = settings.DEBUG
        settings.DEBUG = False
        request = self.factory.post(
            self.str_uri, {'From': '+13333333333'},
            HTTP_X_TWILIO_SIGNATURE=self.sig_with_from_field_blacklisted_caller)
        response = str_view(request)
        r = Response()
        r.reject()
        self.assertEquals(response.content, str(r))
        settings.DEBUG = True
        request = self.factory.post(
            self.str_uri, {'From': '+13333333333'},
            HTTP_X_TWILIO_SIGNATURE=self.sig_with_from_field_blacklisted_caller)
        response = str_view(request)
        r = Response()
        r.reject()
        self.assertEquals(response.content, str(r))
        settings.DEBUG = debug_orig

    def test_decorator_modifies_str(self):
        request = self.factory.post(
            self.str_uri,
            HTTP_X_TWILIO_SIGNATURE=self.str_signature)
        self.assertTrue(isinstance(str_view(request), HttpResponse))

    def test_decorator_modifies_verb(self):
        request = self.factory.post(
            self.verb_uri, HTTP_X_TWILIO_SIGNATURE=self.verb_signature)
        self.assertTrue(isinstance(verb_view(request), HttpResponse))

    def test_decorator_preserves_httpresponse(self):
        request = self.factory.post(
            self.response_uri, HTTP_X_TWILIO_SIGNATURE=self.response_signature)
        self.assertTrue(isinstance(response_view(request), HttpResponse))

    def test_override_forgery_protection_off_debug_off(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=False, DEBUG=False):
            request = self.factory.post(self.str_uri)
            self.assertEquals(str_view(request).status_code, 200)

    def test_override_forgery_protection_off_debug_on(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=False, DEBUG=True):
            request = self.factory.post(self.str_uri)
            self.assertEquals(str_view(request).status_code, 200)

    def test_override_forgery_protection_on_debug_off(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=True, DEBUG=False):
            request = self.factory.post(self.str_uri)
            self.assertEquals(str_view(request).status_code, 403)

    def test_override_forgery_protection_on_debug_on(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=True, DEBUG=True):
            request = self.factory.post(self.str_uri)
            self.assertEquals(str_view(request).status_code, 403)



# -*- coding: utf-8 -*-
"""Useful utility functions."""

import os

from django.http import HttpResponse
from django.conf import settings

from twilio.twiml import Response

from django_twilio.models import Caller, Credential


def discover_twilio_creds(user=None):
    """ Due to the multiple ways of providing SID / AUTH tokens through
        this package, this function will search in the various places that
        credentials might be stored.

        The order this is done in is:

        1. If a User is passed: the keys linked to the
           user model from the Credentials model in the database.
        2. Environment variables
        3. django.conf settings

        We recommend using enviornment variables were possible, it is the
        most secure option

    """

    SID = 'TWILIO_ACCOUNT_SID'
    AUTH = 'TWILIO_AUTH_TOKEN'

    if user:
        creds = Credential.objects.filter(user=user.id)
        if creds.exists():
            creds = creds[0]
            return (creds.account_sid, creds.auth_token)

    if SID and AUTH in os.environ:
        return (os.environ[SID], os.environ[AUTH])

    return (settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN)


def get_blacklisted_response(request):
    """Analyze the incoming Twilio request to determine whether or not to
    reject services. We'll only reject services if the user requesting service
    is on our blacklist.

    :param obj request: The Django HttpRequest object to analyze.
    :rtype: HttpResponse.
    :returns: HttpResponse if the user requesting services is blacklisted, None
        otherwise.
    """
    try:
        caller = Caller.objects.get(phone_number=request.REQUEST['From'])
        if caller.blacklisted:
            r = Response()
            r.reject()
            return HttpResponse(str(r), content_type='application/xml')
    except (KeyError, Caller.DoesNotExist):
        pass

    return None



from django.conf.urls import patterns, url


# Test URLs for our ``django_twilio.decorators`` module.
urlpatterns = patterns(
    'django_twilio.tests.views',
    url(r'^tests/decorators/response_view/$', 'response_view'),
    url(r'^tests/decorators/str_view/$', 'str_view'),
    url(r'^tests/decorators/verb_view/$', 'verb_view'),
)



"""This module provides useful django admin hooks that allow you to manage
various components through the django admin panel (if enabled).
"""

from django.contrib import admin

from django_twilio.models import Caller, Credential


class CallerAdmin(admin.ModelAdmin):
    """This class provides admin panel integration for our
    :class:`django_twilio.models.Caller` model.
    """
    list_display = ('__unicode__', 'blacklisted')


admin.site.register(Caller, CallerAdmin)
admin.site.register(Credential)



# -*- coding: utf-8 -*-

"""Twilio REST client helpers."""
from django_twilio import settings

from twilio.rest import TwilioRestClient

twilio_client = TwilioRestClient(
    settings.TWILIO_ACCOUNT_SID,
    settings.TWILIO_AUTH_TOKEN, version='2010-04-01')



# -*- coding: utf-8 -*-

"""Useful decorators."""


from functools import wraps

from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from django.http import (
    HttpRequest, HttpResponse, HttpResponseForbidden, HttpResponseNotAllowed)

from twilio.twiml import Verb
from twilio.util import RequestValidator

from django_twilio import settings as django_twilio_settings
from django_twilio.utils import get_blacklisted_response


def twilio_view(f):
    """This decorator provides several helpful shortcuts for writing Twilio
    views.

        - It ensures that only requests from Twilio are passed through. This
          helps protect you from forged requests.

        - It ensures your view is exempt from CSRF checks via Django's
          @csrf_exempt decorator. This is necessary for any view that accepts
          POST requests from outside the local domain (eg: Twilio's servers).

        - It enforces the blacklist. If you've got any ``Caller``s who are
          blacklisted, any requests from them will be rejected.

        - It allows your view to (optionally) return TwiML to pass back to
          Twilio's servers instead of building a ``HttpResponse`` object
          manually.

        - It allows your view to (optionally) return any ``twilio.Verb`` object
          instead of building a ``HttpResponse`` object manually.

          .. note::
            The forgery protection checks ONLY happen if ``settings.DEBUG =
            False`` (aka, your site is in production).

    Usage::

        from twilio.twiml import Response

        @twilio_view
        def my_view(request):
            r = Response()
            r.message('Thanks for the SMS message!')
            return r
    """
    @csrf_exempt
    @wraps(f)
    def decorator(request_or_self, methods=['POST'],
                  blacklist=True, *args, **kwargs):

        class_based_view = not(isinstance(request_or_self, HttpRequest))
        if not class_based_view:
            request = request_or_self
        else:
            assert len(args) >= 1
            request = args[0]

        # Turn off Twilio authentication when explicitly requested, or in debug mode.
        # Otherwise things do not work properly. For more information see the docs.
        use_forgery_protection = (
            getattr(settings, 'DJANGO_TWILIO_FORGERY_PROTECTION', not settings.DEBUG))
        if use_forgery_protection:

            if request.method not in methods:
                return HttpResponseNotAllowed(request.method)

            # Forgery check
            try:
                validator = RequestValidator(
                    django_twilio_settings.TWILIO_AUTH_TOKEN)
                url = request.build_absolute_uri()
                signature = request.META['HTTP_X_TWILIO_SIGNATURE']
            except (AttributeError, KeyError):
                return HttpResponseForbidden()

            if request.method == 'POST':
                if not validator.validate(url, request.POST, signature):
                    return HttpResponseForbidden()
            if request.method == 'GET':
                if not validator.validate(url, request.GET, signature):
                    return HttpResponseForbidden()

        # Blacklist check
        if blacklist:
            blacklisted_resp = get_blacklisted_response(request)
            if blacklisted_resp:
                return blacklisted_resp

        response = f(request_or_self, *args, **kwargs)

        if isinstance(response, str):
            return HttpResponse(response, content_type='application/xml')
        elif isinstance(response, Verb):
            return HttpResponse(str(response), content_type='application/xml')
        else:
            return response
    return decorator



from django.test import TestCase
from django.contrib.auth.models import User

from twilio.rest import TwilioRestClient

from django_twilio.client import twilio_client
from django_twilio.models import Credential
from django_twilio import settings
from django_twilio.utils import discover_twilio_creds


class TwilioClientTestCase(TestCase):

    def test_twilio_client_exists(self):
        self.assertIsInstance(twilio_client, TwilioRestClient)

    def test_twilio_client_sets_creds(self):
        self.assertEqual(
            twilio_client.auth,
            (settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN))

    def test_twilio_client_with_creds_model(self):
        self.user = User.objects.create(username='test', password='pass')
        self.creds = Credential.objects.create(
            name='Test Creds',
            account_sid='AAA',
            auth_token='BBB',
            user=self.user,
        )

        deets = discover_twilio_creds(user=self.user)

        self.assertEquals(deets[0], self.creds.account_sid)
        self.assertEquals(deets[1], self.creds.auth_token)



# -*- coding: utf-8 -*-

from django.db import models
from django.conf import settings

from phonenumber_field.modelfields import PhoneNumberField

from django_twilio.compat import AUTH_USER_MODEL


class Caller(models.Model):
    """ A caller is defined uniquely by their phone number.

    :param bool blacklisted: Designates whether the caller can use our
        services.
    :param char phone_number: Unique phone number in `E.164
        <http://en.wikipedia.org/wiki/E.164>`_ format.

    """
    blacklisted = models.BooleanField()
    phone_number = PhoneNumberField(unique=True)

    def __unicode__(self):
        name = str(self.phone_number)
        if self.blacklisted:
            name += ' (blacklisted)'
        return name


class Credential(models.Model):
    """ A Credential model is a set of SID / AUTH tokens for the Twilio.com API

        The Credential model can be used if a project uses more than one
        Twilio account, or provides Users with access to Twilio powered
        web apps that need their own custom credentials.

    :param char name: The name used to distinguish this credential
    :param char account_sid: The Twilio account_sid
    :param char auth_token: The Twilio auth_token
    :param key user: The user linked to this Credential

    """

    def __unicode__(self):
        return ' '.join([self.name, '-', self.account_sid])

    name = models.CharField(max_length=30)

    user = models.OneToOneField(AUTH_USER_MODEL)

    account_sid = models.CharField(max_length=34)

    auth_token = models.CharField(max_length=32)



from django.conf import settings
import django

__all__ = ['User', 'AUTH_USER_MODEL']

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


# Django 1.5+ compatibility
if django.VERSION >= (1, 5):
    from django.contrib.auth import get_user_model
    User = get_user_model()
    username_field = User.USERNAME_FIELD
else:
    from django.contrib.auth.models import User
    username_field = 'username'



from types import MethodType

from django.test import TestCase
from django.contrib.auth.models import User
from django_twilio.models import Caller, Credential


class CallerTestCase(TestCase):
    """Run tests against the :class:`django_twilio.models.Caller` model ."""

    def setUp(self):
        self.caller = Caller.objects.create(
            phone_number='12223334444', blacklisted=False)

    def test_has_unicode(self):
        self.assertTrue(isinstance(self.caller.__unicode__, MethodType))

    def test_unicode_returns_str(self):
        self.assertTrue(isinstance(self.caller.__unicode__(), str))

    def test_unicode_doesnt_contain_blacklisted(self):
        self.assertFalse('blacklisted' in self.caller.__unicode__())

    def test_unicode_contains_blacklisted(self):
        self.caller.blacklisted = True
        self.caller.save()
        self.assertTrue('blacklisted' in self.caller.__unicode__())

    def tearDown(self):
        self.caller.delete()


class CredentialTests(TestCase):

    def setUp(self):
        self.user = User.objects.create(username='test', password='pass')
        self.creds = Credential.objects.create(
            name='Test Creds',
            account_sid='XXX',
            auth_token='YYY',
            user=self.user,
        )

    def test_unicode(self):
        ''' Assert that unicode renders how we'd like it too '''
        self.assertEquals(self.creds.__unicode__(), 'Test Creds - XXX')

    def test_credentials_fields(self):
        ''' Assert the fields are working correctly '''
        self.assertEquals(self.creds.name, 'Test Creds')
        self.assertEquals(self.creds.account_sid, 'XXX')
        self.assertEquals(self.creds.auth_token, 'YYY')
        self.assertEquals(self.creds.user, self.user)



from hmac import new
from hashlib import sha1
from base64 import encodestring

import os

from django.http import HttpResponse
from django.test import Client, RequestFactory, TestCase
from twilio.twiml import Response

from django_twilio import settings
from django_twilio.decorators import twilio_view
from django_twilio.views import (
    conference, dial, gather, play, record, say, sms)


@twilio_view
def response_view(request):
    """A simple test view that returns a HttpResponse object."""
    return HttpResponse(
        '<Response><Message>Hello from Django</Message></Response>',
        content_type='text/xml')


@twilio_view
def str_view(request):
    """A simple test view that returns a string."""
    return '<Response><Message>Hi!</Message></Response>'


@twilio_view
def verb_view(request):
    """A simple test view that returns a ``twilio.Verb`` object."""
    r = Response()
    r.reject()
    return r


class SayTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.say_uri = '/tests/views/say/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate Twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/say/' % self.uri, sha1).digest()).strip()

    def test_say_no_text(self):
        request = self.factory.post(
            self.say_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertRaises(TypeError, say, request)

    def test_say_with_text(self):
        request = self.factory.post(
            self.say_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(say(request, text='hi').status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']


class PlayTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.play_uri = '/tests/views/play/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/play/' % self.uri, sha1).digest()).strip()

    def test_play_no_url(self):
        request = self.factory.post(
            self.play_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertRaises(TypeError, play, request)

    def test_play_with_url(self):
        request = self.factory.post(
            self.play_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(
            play(request, url='http://b.com/b.wav').status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']


class GatherTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.gather_uri = '/tests/views/gather/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/gather/' % self.uri, sha1).digest()).strip()

    def test_gather(self):
        request = self.factory.post(
            self.gather_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(gather(request).status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']


class RecordTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.record_uri = '/tests/views/record/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/record/' % self.uri, sha1).digest()).strip()

    def test_record(self):
        request = self.factory.post(
            self.record_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(record(request).status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']


class SmsTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.sms_uri = '/tests/views/sms/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/sms/' % self.uri, sha1).digest()).strip()

    def test_sms_no_message(self):
        request = self.factory.post(
            self.sms_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertRaises(TypeError, sms, request)

    def test_sms_with_message(self):
        request = self.factory.post(
            self.sms_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(sms(request, message='test').status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']


class DialTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.dial_uri = '/tests/views/dial/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/dial/' % self.uri, sha1).digest()).strip()

    def test_dial_no_number(self):
        request = self.factory.post(
            self.dial_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertRaises(TypeError, dial, request)

    def test_dial_with_number(self):
        request = self.factory.post(
            self.dial_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(dial(
            request, number='+18182223333').status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']


class ConferenceTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()

        # Test URIs.
        self.uri = 'http://testserver/tests/views'
        self.conf_uri = '/tests/views/conference/'

        # Guarantee a value for the required configuration settings after each
        # test case.
        settings.TWILIO_ACCOUNT_SID = 'xxx'
        settings.TWILIO_AUTH_TOKEN = 'xxx'

        # Pre-calculate twilio signatures for our test views.
        self.signature = encodestring(
            new(settings.TWILIO_AUTH_TOKEN,
                '%s/conference/' % self.uri, sha1).digest()).strip()

    def test_conference_no_name(self):
        request = self.factory.post(
            self.conf_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertRaises(TypeError, conference, request)

    def test_conference_with_name(self):
        request = self.factory.post(
            self.conf_uri, HTTP_X_TWILIO_SIGNATURE=self.signature)
        self.assertEquals(conference(request, name='a').status_code, 200)

    def tearDown(self):
        settings.TWILIO_ACCOUNT_SID = os.environ['TWILIO_ACCOUNT_SID']
        settings.TWILIO_AUTH_TOKEN = os.environ['TWILIO_AUTH_TOKEN']



from os.path import abspath, dirname, join, normpath

from setuptools import find_packages, setup


setup(

    # Basic package information:
    name = 'django-twilio',
    version = '0.5.2',
    packages = find_packages(),

    # Packaging options:
    zip_safe = False,
    include_package_data = True,

    # Package dependencies:
    install_requires = ['twilio>=3.3.6', 'Django>=1.3.1', 'django-phonenumber-field>=0.2a3'],

    # Metadata for PyPI:
    author = 'Randall Degges',
    author_email = 'rdegges@gmail.com',
    license = 'UNLICENSE',
    url = 'http://twilio.com/',
    keywords = 'twilio telephony call phone voip sms',
    description = 'Build Twilio functionality into your Django apps.',
    long_description = open(normpath(join(dirname(abspath(__file__)),
        'README.rst'))).read()

)
